(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.VueRoast = factory());
}(this, (function () { 'use strict';

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var _default =
/*#__PURE__*/
function () {
  function _default(ttl) {
    var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Function.prototype;

    _classCallCheck(this, _default);

    this.ttl = ttl;
    this.action = action;
  }

  _createClass(_default, [{
    key: "start",
    value: function start() {
      var ttl = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.ttl;
      this.timer = setTimeout(this.action, ttl);
    }
  }, {
    key: "stop",
    value: function stop() {
      clearTimeout(this.timer);
    }
  }, {
    key: "restart",
    value: function restart() {
      var ttl = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.ttl;
      this.stop();
      this.start(ttl);
    }
  }]);

  return _default;
}();

var ToastMessage = {
  render: function render() {
    var _vm = this;

    var _h = _vm.$createElement;

    var _c = _vm._self._c || _h;

    return _c('section', {
      staticClass: "toast-message",
      class: _vm.level,
      attrs: {
        "role": "alert"
      },
      on: {
        "mouseenter": function mouseenter($event) {
          _vm.timer.stop();
        },
        "mouseleave": function mouseleave($event) {
          _vm.timer.restart(_vm.timer.ttl / 2);
        }
      }
    }, [_c('div', {
      staticClass: "toast-header"
    }, [_c('span', {
      staticClass: "title bigger"
    }, [_vm._v(_vm._s(_vm.title))]), _vm._v(" "), _c('span', {
      staticClass: "close-btn smaller",
      attrs: {
        "role": "button"
      },
      on: {
        "click": _vm.close
      }
    }, [_vm._v("✖")])]), _vm._v(" "), _c('div', {
      staticClass: "toast-body"
    }, [_c('span', [_vm._v(_vm._s(_vm.content))]), _vm._v(" "), _vm.options.action ? _c('span', {
      attrs: {
        "role": "button"
      },
      on: {
        "click": _vm.options.action.method
      }
    }, [_vm._v(" " + _vm._s(_vm.options.action.placeholder) + " ")]) : _vm._e()]), _vm._v(" "), _c('div', {
      staticClass: "toast-footer"
    }, [_c('span', {
      staticClass: "smaller"
    }, [_vm._v(_vm._s(_vm.footer))])])]);
  },
  staticRenderFns: [],
  props: {
    id: {
      type: Number,
      required: true
    },
    title: {
      type: String,
      default: ''
    },
    body: {
      type: String,
      default: ''
    },
    footer: {
      type: String,
      default: ''
    },
    level: {
      type: String,
      required: true
    },
    options: {
      action: {
        placeholder: String,
        method: Function
      },
      truncateAfter: {
        type: Number,
        required: false
      }
    },
    timer: {
      type: Object,
      required: true
    }
  },
  computed: {
    content: function content() {
      var truncateAfter = this.options.truncateAfter;
      return truncateAfter ? this.body : this.body.substr(0, truncateAfter);
    }
  },
  methods: {
    close: function close() {
      this.$emit('close', this.id);
    }
  },
  mounted: function mounted() {
    this.timer.action = this.close;
    this.timer.start();
  }
};

var ToastContainer = {
  render: function render() {
    var _vm = this;

    var _h = _vm.$createElement;

    var _c = _vm._self._c || _h;

    return _c('section', {
      staticClass: "toast-container",
      class: _vm.position
    }, [_c('transition-group', {
      attrs: {
        "name": _vm.transition
      }
    }, _vm._l(_vm.toasts, function (_ref) {
      var id = _ref.id,
          message = _ref.message,
          timer = _ref.timer,
          options = _ref.options;
      return _c('toast-message', _vm._b({
        key: id,
        attrs: {
          "id": id,
          "options": options,
          "timer": timer
        },
        on: {
          "close": _vm.close
        }
      }, 'toast-message', message, false));
    }))], 1);
  },
  staticRenderFns: [],
  props: {
    position: {
      type: String,
      required: true
    },
    transition: {
      type: String,
      required: true
    },
    toasts: {
      type: Array,
      default: function _default() {
        return [];
      }
    }
  },
  methods: {
    close: function close(id) {
      var index = this.toasts.findIndex(function (el) {
        return el.id === id;
      });
      this.toasts.splice(index, 1);
    }
  },
  components: {
    ToastMessage: ToastMessage
  }
};

var DEFAULT_POSITION = 'bottom-left';
var DEFAULT_TTL = 5000;
var DEFAULT_THRESHOLD = 3;
var cache = [];
var defaults = {};

function getTransition(position) {
  if (position === 'top-middle') return 'slide-fade-down';
  if (position === 'bottom-middle') return 'slide-fade-up';
  if (position.includes('right')) return 'slide-fade-left';
  return 'slide-fade-right';
}

function setDefaults() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var ttl = config.ttl || DEFAULT_TTL;
  var threshold = config.threshold || DEFAULT_THRESHOLD;
  var position = config.positionClass || DEFAULT_POSITION;
  var transition = getTransition(position);
  var pushAction = position.includes('top') ? 'unshift' : 'push';
  Object.assign(defaults, {
    ttl: ttl,
    threshold: threshold,
    position: position,
    transition: transition,
    pushAction: pushAction
  });
}

function uniqueId(exclude) {
  var id = Math.ceil(Date.now() * Math.random());
  if (exclude.includes(id)) return uniqueId(exclude);
  return id;
}


/**
 * @param {Object} message - toast message
 * @param {Object} options - toast options
 * @param {number} options.ttl - time to live in milliseconds
 * @param {number} options.truncateAfter - max message char length to display
 * @param {string} options.action.placeholder - toast click action placeholder
 * @param {Function} options.action.method - method to be invoked on click
 * @returns {undefined}
 */

function toast() {
  var message = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  if (!message.title && !message.body && !message.footer) return;
  if (cache.length === defaults.threshold) cache.shift();
  options.ttl = options.ttl || defaults.ttl;
  var id = uniqueId(cache.map(function (el) {
    return el.id;
  }));
  var timer = new _default(options.ttl);
  cache[defaults.pushAction]({
    id: id,
    message: message,
    options: options,
    timer: timer
  });
}


var index = {
  install: function install(Vue) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    setDefaults(config);
    var Container = Vue.extend(ToastContainer);
    var position = defaults.position,
        transition = defaults.transition;
    var container = new Container({
      propsData: {
        position: position,
        transition: transition,
        toasts: cache
      }
    }).$mount();
    document.body.appendChild(container.$el);
    Vue.toast = toast;
    Vue.prototype.$toast = toast;
  }
};

return index;

})));
